==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_base/lib/corelibs/superset/sep_dff_struct.c <==
#define SET_SDOC 1
/* initial values for these pointers is zero (no streams initialized) */
#include "../include/superset_internal.h"
#ifndef YES
#define YES 1
#endif
#ifndef NO
#define NO 0
#endif
#include <superset.h>

static sep_dff *sepdfflist=SEPNULL;
static sep_dff *sepdfftail=SEPNULL;

/*
-------------------------------------------------

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Sun Aug 16 13:52:22 PDT 1998

Purpose: 

*/	 

#include<superset.h> 
#include"../include/superset_internal.h" 

/*<
tag_info_sepdff

Usage
sepdff=tag_info_sepdff(sepdffname,usage)

Input Paramters
sepdffname   -   char*         pointer to sep_dff structure  
usage       -   usage_type    usage (INPUT,OUTPUT,SCRATCH) for structure
Output Paramters

Return Values

Description
Finds and initializes a tag

>*/




#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff* tag_info_sepdff(char *name, enum usage_type type)
_XFUNCPROTOEND
#else
sep_dff* tag_info_sepdff(name,type) 
char *name ;
enum usage_type type;
#endif 
{ 
 sep_dff *curr;

    curr =SEPDFF_head();


		
    while( curr != SEPNULL ){
    if( strcmp(name,curr->name) == 0 ){
      return curr;
  }
  curr = curr->next;
    }


   if(type==INQUIRE) return(SEPNULL);

/* fell through so we must create it and put it at the end of the list*/
    curr = SEPDFF_new( name, type );
		SEPDFF_addend(curr);

    switch( curr->usage ){
     case INPUT:
        break;
     case OUTPUT:
        break;
     case SCRATCH:
/*				if(NULL==auxinout(name))*/
/*					seperr("trouble opening %s as tagsrc \n",name);*/
        break;
		 case INQUIRE:
				return(SEPNULL);
				break;
    }

    return curr;
}

/*  $Id: sep_dff_struct.c,v 1.1.1.1 2004/03/25 06:37:24 cvs Exp $ */
#define SET_SDOC 1
/*

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Sun Jun 16 14:02:50 PDT 1999

Purpose: 

*/	 




/*<
sepdff_head


Usage
sep_dff *sepdff_head(void)

Return Values
sep_dff   -  first sep_dff in list 

Description

Return the begining of a sep_dff list
>*/



#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff * SEPDFF_head(void) 
_XFUNCPROTOEND
#else
sep_dff * SEPDFF_head() 
#endif 
{ return(sepdfflist); }

/*<
SEPDFF_del


Usage 
sep_dff *SEPDFF_del(sep_dff *cur)

Input Paramters
cur  -  sepdff*    sep_dff structure to delete

Output Paramters

Return Values
Returns a pointer to the created structure

Description

Delete structure from list
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_del( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_del( curr )
     sep_dff *curr;
#endif
{
    sep_dff *tmp1,*tmp2;

    tmp1=curr->prev;
    tmp2=curr->next;

    if( curr == sepdfflist ) sepdfflist = tmp2;
    if( curr == sepdfftail ) sepdfftail = tmp1;

    if( tmp1 != 0 ) tmp1->next = tmp2;
    if( tmp2 != 0 ) tmp2->prev = tmp1;


}

/*<
sepdff_start


Usage 
sep_dff *SEPDFF_addstart(curr)

Input Paramters
curr  -  sepdff*  sep_dff structure to add

Output Paramters

Return Values
Returns a pointer to the first structure

Description
Start a sep_dff list
>*/


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_addstart( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_addstart( curr )
sep_dff *curr;
#endif
{
    if( curr == sepdfflist )  return;

    curr->prev = 0;
    if( sepdfflist != 0 ) sepdfflist->prev=curr;
    curr->next=sepdfflist;
    sepdfflist=curr;
    if( sepdfftail == 0 ) sepdfftail=sepdfflist; /* the first entry */
}


/*<
SEPDFF_addend


Usage 
sep_dff *SEPDFF_addend(name)

Input Paramters
curr  -  sepdff*  sep_dff structure to add

Output Paramters

Return Values
Pointer to the structure

Description

Add entry to end of sep_dff list
>*/
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_addend( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_addend( curr )
     sep_dff *curr;
#endif
{
    if( curr == sepdfftail )  return;

    curr->prev=sepdfftail;
    if( sepdfftail != 0 ) sepdfftail->next = curr;
    curr-> next = 0;
    sepdfftail=curr;
    if( sepdfflist == 0 ) sepdfflist = curr;
}

 
/*<
sepdff_new


Usage 
sep_dff *SEPDFF_new(name, usage)

Input Paramters
curr  -  sepdff*       sep_dff structure to add
usage -  usage_type   purpose (INPUT,OUTPUT,SCRATCH) for type

Output Paramters

Return Values
pointer to the structure

Description

Add entry to sep_dff list
>*/
 
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff *SEPDFF_new( char *tag, enum usage_type usage )
_XFUNCPROTOEND
#else
sep_dff * SEPDFF_new( tag, usage )
     char *tag; 
enum usage_type usage;
#endif
{
  sep_dff *newinf;
	int i;

  /* temporary buffer for strings from par line, max length=1024 */
  char tmppar[1024];

  /* if this is just a query, generate an error */
  if( usage == INQUIRE ){ 
     seperr("Unknown tag \"%s\" \n",tag ); 
   }

  newinf = (sep_dff*) alloc(sizeof(sep_dff));

  newinf->name = (char*)alloc(1+(int)strlen(tag));
  strcpy(newinf->name,tag);


	newinf->usage=usage;
  newinf->nmpi=1;
  newinf->impi=0;
  newinf->axis_convert=SEPNULL;
  newinf->nsect=0;
  newinf->nsect_ithread=1;
  newinf->sect_ithread=SEPNULL;
  newinf->sect_axis=-1;
  newinf->comm=NONE;
  newinf->pattern=NODEF;
  newinf->sect_thread=SEPNULL;
  newinf->sect_thread=SEPNULL;
  newinf->tag_sect=SEPNULL; 
  newinf->mach_sect=SEPNULL; 
  newinf->master_data=1; 

 
return(newinf);
} 


/*<
SEPDFF_print


Usage 
void SEPDFF_print(sepdff)

Input Paramters
info   -  sepdff*  sep_dff structure to print information about

Output Paramters

Return Values

Description

Print information in sepdff
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_print( sep_dff *info )
_XFUNCPROTOEND
#else
void SEPDFF_print( info )
sepdff* info;
#endif
{
int i1,i;



if(info->nsect ==0) sepwarn(0,"Not spread over processors\n");
else{
 sepwarn(0,"Data spread into %d parts, ",info->nsect);
   sepwarn(0,"%d ",info->sect_axis);
 sepwarn(0,"\n");
 if(info->nmpi >=0){
     sepwarn(0,"I am thread  %d of  %d processors\n",info->impi,info->nmpi); 
     sepwarn(0,"I have %d section alone these axis/axes \n",
        info->nsect_ithread);
  }
  else{
    sepwarn(0,"Not running MPI");
      for(i=0; i < info->nsect; i++){
       sepwarn(0,"Section %d tag=%s  originally on=%s \n",i,
          info->tag_sect[info->sect_ithread[i]],
          info->mach_sect[info->sect_ithread[i]]);
    }
  }
}
sepwarn(0,"Other Information \n --------------------------------------------\n");

}
_XFUNCPROTOBEGIN
int SEPDFF_copy( sep_dff *from, sep_dff *to)
_XFUNCPROTOEND
{
 int i;

  if(from->nsect>0){
    if(0!=SEPDFF_set_nsect(to,from->nsect))
      return(sepwarn(NOT_MET,"trouble setting nsect for tag %s \n",to->name));
  
    if(SEPNULL!=from->mach_sect){
      for(i=0; i < from->nsect; i++){
        if(0!=SEPDFF_set_mach_sect(to,i,from->mach_sect[i]))
          return(sepwarn(NOT_MET,"trouble setting mach_sect %d for tag %s \n",i,to->name));
      }
    }
    if(SEPNULL!=from->tag_sect){
      for(i=0; i < from->nsect; i++){
        if(0!=SEPDFF_set_tag_sect(to,i,from->tag_sect[i]))
         return(sepwarn(NOT_MET,"trouble setting tag_sect %d for tag %s \n",i,to->name));
      }
    }
    if(SEPNULL!=from->sect_thread) memcpy((void*)to->sect_thread,(const void*)from->sect_thread,
       from->nsect*sizeof(int));

    if(SEPNULL!=from->wind_sect_nh) memcpy((void*)to->wind_sect_nh,(const void*)from->wind_sect_nh,
       from->nsect*sizeof(int));
  }

  if(from->naxis_convert >0){
    to->sect_axis=from->sect_axis;
    if(0!=SEPDFF_set_sect_axis(to,from->sect_axis,from->naxis_convert))
       return(sepwarn(NOT_MET,"troublet setting sct axis for %s \n",to->name));

    if(from->axis_convert!=SEPNULL)  memcpy((void*)to->axis_convert,
     (const void*)from->axis_convert, from->naxis_convert*sizeof(int));
  } 

  to->comm=from->comm;
  to->pattern=from->pattern;
  if(0!=SEPDFF_del_axis_thread_info(to))
    return(sepwarn(NOT_MET,"trouble deleting trhead info for dff %s \n",to->name));
  if(0!=SEPDFF_set_axis_thread_info(to))
    return(sepwarn(NOT_MET,"trouble setting axis info for dff %s \n",to->name));
/*  to->nsect_ithread=from->nsect_ithread;*/
/*  if(from->sect_ithread!=SEPNULL)  memcpy((void*)to->sect_ithread,(const void*)from->sect_ithread,*/
/*    from->nsect_ithread*sizeof(int));*/

  to->master_data=from->master_data;
  return(SUCCESS);
}
                                                                                          

/*  $Id: sep_dff_struct.c,v 1.1.1.1 2004/03/25 06:37:24 cvs Exp $ */
_XFUNCPROTOBEGIN
int SEPDFF_delete( sep_dff *info )
_XFUNCPROTOEND
{

if(0!=SEPDFF_clean(info))
  return(sepwarn(NOT_MET,"trouble cleaning structure %s\n", info->name ));

free(info->name);
SEPDFF_del(info);

return(0);

}
_XFUNCPROTOBEGIN
int SEPDFF_clean( sep_dff *info )
_XFUNCPROTOEND
{
 int i;
  if(info->nsect >0){
    for(i=0; i< info->nsect; i++) sep3d_delete(info->tag_sect[i]);
  }
 if(info->nsect>0) {
 if(0!=SEPDFF_del_axis_thread_info(info) ||
   0!=SEPDFF_del_axis_convert(info) || 0!=SEPDFF_del_nsect(info))
     return(sepwarn(NOT_MET, "trouble cleaning mpi info tag=%s \n",info->name));
}
 
 

return(0);
}
                                                                                          

/*  $Id: sep_dff_struct.c,v 1.1.1.1 2004/03/25 06:37:24 cvs Exp $ */

==> seplib-6.3.9.tar.gz: seplib-6.3.9/seplib_base/lib/corelibs/superset/sep_dff_struct.c <==
#define SET_SDOC 1
/* initial values for these pointers is zero (no streams initialized) */
#include "superset_internal.h"
#ifndef YES
#define YES 1
#endif
#ifndef NO
#define NO 0
#endif
#include <superset.h>

static sep_dff *sepdfflist=SEPNULL;
static sep_dff *sepdfftail=SEPNULL;

/*
-------------------------------------------------

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Sun Aug 16 13:52:22 PDT 1998

Purpose: 

*/	 

#include<superset.h> 
#include"../include/superset_internal.h" 

/*<
tag_info_sepdff

Usage
sepdff=tag_info_sepdff(sepdffname,usage)

Input Paramters
sepdffname   -   char*         pointer to sep_dff structure  
usage       -   usage_type    usage (INPUT,OUTPUT,SCRATCH) for structure
Output Paramters

Return Values

Description
Finds and initializes a tag

>*/




#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff* tag_info_sepdff(char *name, enum usage_type type)
_XFUNCPROTOEND
#else
sep_dff* tag_info_sepdff(name,type) 
char *name ;
enum usage_type type;
#endif 
{ 
 sep_dff *curr;

    curr =SEPDFF_head();


		
    while( curr != SEPNULL ){
    if( strcmp(name,curr->name) == 0 ){
      return curr;
  }
  curr = curr->next;
    }


   if(type==INQUIRE) return(SEPNULL);

/* fell through so we must create it and put it at the end of the list*/
    curr = SEPDFF_new( name, type );
		SEPDFF_addend(curr);

    switch( curr->usage ){
     case INPUT:
        break;
     case OUTPUT:
        break;
     case SCRATCH:
/*				if(NULL==auxinout(name))*/
/*					seperr("trouble opening %s as tagsrc \n",name);*/
        break;
		 case INQUIRE:
				return(SEPNULL);
				break;
    }

    return curr;
}

/*  $Id: sep_dff_struct.c,v 1.2 2004/04/08 22:32:28 bob Exp $ */
#define SET_SDOC 1
/*

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Sun Jun 16 14:02:50 PDT 1999

Purpose: 

*/	 




/*<
sepdff_head


Usage
sep_dff *sepdff_head(void)

Return Values
sep_dff   -  first sep_dff in list 

Description

Return the begining of a sep_dff list
>*/



#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff * SEPDFF_head(void) 
_XFUNCPROTOEND
#else
sep_dff * SEPDFF_head() 
#endif 
{ return(sepdfflist); }

/*<
SEPDFF_del


Usage 
sep_dff *SEPDFF_del(sep_dff *cur)

Input Paramters
cur  -  sepdff*    sep_dff structure to delete

Output Paramters

Return Values
Returns a pointer to the created structure

Description

Delete structure from list
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_del( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_del( curr )
     sep_dff *curr;
#endif
{
    sep_dff *tmp1,*tmp2;

    tmp1=curr->prev;
    tmp2=curr->next;

    if( curr == sepdfflist ) sepdfflist = tmp2;
    if( curr == sepdfftail ) sepdfftail = tmp1;

    if( tmp1 != 0 ) tmp1->next = tmp2;
    if( tmp2 != 0 ) tmp2->prev = tmp1;


}

/*<
sepdff_start


Usage 
sep_dff *SEPDFF_addstart(curr)

Input Paramters
curr  -  sepdff*  sep_dff structure to add

Output Paramters

Return Values
Returns a pointer to the first structure

Description
Start a sep_dff list
>*/


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_addstart( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_addstart( curr )
sep_dff *curr;
#endif
{
    if( curr == sepdfflist )  return;

    curr->prev = 0;
    if( sepdfflist != 0 ) sepdfflist->prev=curr;
    curr->next=sepdfflist;
    sepdfflist=curr;
    if( sepdfftail == 0 ) sepdfftail=sepdfflist; /* the first entry */
}


/*<
SEPDFF_addend


Usage 
sep_dff *SEPDFF_addend(name)

Input Paramters
curr  -  sepdff*  sep_dff structure to add

Output Paramters

Return Values
Pointer to the structure

Description

Add entry to end of sep_dff list
>*/
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_addend( sep_dff *curr )
_XFUNCPROTOEND
#else
void SEPDFF_addend( curr )
     sep_dff *curr;
#endif
{
    if( curr == sepdfftail )  return;

    curr->prev=sepdfftail;
    if( sepdfftail != 0 ) sepdfftail->next = curr;
    curr-> next = 0;
    sepdfftail=curr;
    if( sepdfflist == 0 ) sepdfflist = curr;
}

 
/*<
sepdff_new


Usage 
sep_dff *SEPDFF_new(name, usage)

Input Paramters
curr  -  sepdff*       sep_dff structure to add
usage -  usage_type   purpose (INPUT,OUTPUT,SCRATCH) for type

Output Paramters

Return Values
pointer to the structure

Description

Add entry to sep_dff list
>*/
 
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
sep_dff *SEPDFF_new( char *tag, enum usage_type usage )
_XFUNCPROTOEND
#else
sep_dff * SEPDFF_new( tag, usage )
     char *tag; 
enum usage_type usage;
#endif
{
  sep_dff *newinf;
	int i;

  /* temporary buffer for strings from par line, max length=1024 */
  char tmppar[1024];

  /* if this is just a query, generate an error */
  if( usage == INQUIRE ){ 
     seperr("Unknown tag \"%s\" \n",tag ); 
   }

  newinf = (sep_dff*) alloc(sizeof(sep_dff));

  newinf->name = (char*)alloc(1+(int)strlen(tag));
  strcpy(newinf->name,tag);


	newinf->usage=usage;
  newinf->nmpi=1;
  newinf->impi=0;
  newinf->axis_convert=SEPNULL;
  newinf->nsect=0;
  newinf->nsect_ithread=1;
  newinf->sect_ithread=SEPNULL;
  newinf->sect_axis=-1;
  newinf->comm=NONE;
  newinf->pattern=NODEF;
  newinf->sect_thread=SEPNULL;
  newinf->sect_thread=SEPNULL;
  newinf->tag_sect=SEPNULL; 
  newinf->mach_sect=SEPNULL; 
  newinf->dff_rite=0;
  newinf->master_data=1; 

 
return(newinf);
} 


/*<
SEPDFF_print


Usage 
void SEPDFF_print(sepdff)

Input Paramters
info   -  sepdff*  sep_dff structure to print information about

Output Paramters

Return Values

Description

Print information in sepdff
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void SEPDFF_print( sep_dff *info )
_XFUNCPROTOEND
#else
void SEPDFF_print( info )
sepdff* info;
#endif
{
int i1,i;



if(info->nsect ==0) sepwarn(0,"Not spread over processors\n");
else{
 sepwarn(0,"Data spread into %d parts, ",info->nsect);
   sepwarn(0,"%d ",info->sect_axis);
 sepwarn(0,"\n");
 if(info->nmpi >=0){
     sepwarn(0,"I am thread  %d of  %d processors\n",info->impi,info->nmpi); 
     sepwarn(0,"I have %d section alone these axis/axes \n",
        info->nsect_ithread);
  }
  else{
    sepwarn(0,"Not running MPI");
      for(i=0; i < info->nsect; i++){
       sepwarn(0,"Section %d tag=%s  originally on=%s \n",i,
          info->tag_sect[info->sect_ithread[i]],
          info->mach_sect[info->sect_ithread[i]]);
    }
  }
}
sepwarn(0,"Other Information \n --------------------------------------------\n");

}
_XFUNCPROTOBEGIN
int SEPDFF_copy( sep_dff *from, sep_dff *to)
_XFUNCPROTOEND
{
 int i;

  if(from->nsect>0){
    if(0!=SEPDFF_set_nsect(to,from->nsect))
      return(sepwarn(NOT_MET,"trouble setting nsect for tag %s \n",to->name));
  
    if(SEPNULL!=from->mach_sect){
      for(i=0; i < from->nsect; i++){
        if(0!=SEPDFF_set_mach_sect(to,i,from->mach_sect[i]))
          return(sepwarn(NOT_MET,"trouble setting mach_sect %d for tag %s \n",i,to->name));
      }
    }
    if(SEPNULL!=from->tag_sect){
      for(i=0; i < from->nsect; i++){
        if(0!=SEPDFF_set_tag_sect(to,i,from->tag_sect[i]))
         return(sepwarn(NOT_MET,"trouble setting tag_sect %d for tag %s \n",i,to->name));
      }
    }
    if(SEPNULL!=from->sect_thread) memcpy((void*)to->sect_thread,(const void*)from->sect_thread,
       from->nsect*sizeof(int));

    if(SEPNULL!=from->wind_sect_nh) memcpy((void*)to->wind_sect_nh,(const void*)from->wind_sect_nh,
       from->nsect*sizeof(int));
  }

  if(from->naxis_convert >0){
    to->sect_axis=from->sect_axis;
    if(0!=SEPDFF_set_sect_axis(to,from->sect_axis,from->naxis_convert))
       return(sepwarn(NOT_MET,"troublet setting sct axis for %s \n",to->name));

    if(from->axis_convert!=SEPNULL)  memcpy((void*)to->axis_convert,
     (const void*)from->axis_convert, from->naxis_convert*sizeof(int));
  } 

  to->comm=from->comm;
  to->pattern=from->pattern;
  if(0!=SEPDFF_del_axis_thread_info(to))
    return(sepwarn(NOT_MET,"trouble deleting trhead info for dff %s \n",to->name));
  if(0!=SEPDFF_set_axis_thread_info(to))
    return(sepwarn(NOT_MET,"trouble setting axis info for dff %s \n",to->name));
/*  to->nsect_ithread=from->nsect_ithread;*/
/*  if(from->sect_ithread!=SEPNULL)  memcpy((void*)to->sect_ithread,(const void*)from->sect_ithread,*/
/*    from->nsect_ithread*sizeof(int));*/

  to->master_data=from->master_data;
  return(SUCCESS);
}
                                                                                          

/*  $Id: sep_dff_struct.c,v 1.2 2004/04/08 22:32:28 bob Exp $ */
_XFUNCPROTOBEGIN
int SEPDFF_delete( sep_dff *info )
_XFUNCPROTOEND
{

if(0!=SEPDFF_clean(info))
  return(sepwarn(NOT_MET,"trouble cleaning structure %s\n", info->name ));

free(info->name);
SEPDFF_del(info);

return(0);

}
_XFUNCPROTOBEGIN
int SEPDFF_clean( sep_dff *info )
_XFUNCPROTOEND
{
 int i;
 char temp_ch[2049];
  if(info->nsect >0){
    for(i=0; i< info->nsect; i++){
       SEPDFF_sect_name(info,i,temp_ch); 
       sep3d_delete(temp_ch);
    }
  }
 if(info->nsect>0) {
 if(0!=SEPDFF_del_axis_thread_info(info) ||
   0!=SEPDFF_del_axis_convert(info) || 0!=SEPDFF_del_nsect(info))
     return(sepwarn(NOT_MET, "trouble cleaning mpi info tag=%s \n",info->name));
}
 
 

return(0);
}
                                                                                          

/*  $Id: sep_dff_struct.c,v 1.2 2004/04/08 22:32:28 bob Exp $ */
